@@ -1,16 +1,16 @@
-//  Copyright 2022 Google Inc. All Rights Reserved.
-//
-//  Licensed under the Apache License, Version 2.0 (the "License");
-//  you may not use this file except in compliance with the License.
-//  You may obtain a copy of the License at
-//
-//      http://www.apache.org/licenses/LICENSE-2.0
-//
-//  Unless required by applicable law or agreed to in writing, software
-//  distributed under the License is distributed on an "AS IS" BASIS,
-//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-//  See the License for the specific language governing permissions and
-//  limitations under the License.
+// Copyright 2022 Google LLC
+
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+
+//     https://www.apache.org/licenses/LICENSE-2.0
+
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 
 // GoogleAuthorizedKeys obtains SSH keys from metadata.
 package main
@@ -20,7 +20,6 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/http"
 	"os"
 	"runtime"
@@ -34,7 +33,6 @@ import (
 
 var (
 	programName       = "GoogleAuthorizedKeysCommand"
-	version           string
 	metadataURL       = "http://169.254.169.254/computeMetadata/v1/"
 	metadataRecursive = "/?recursive=true&alt=json"
 	defaultTimeout    = 2 * time.Second
@@ -83,7 +81,7 @@ func getMetadata(key string, recurse bool) ([]byte, error) {
 	}
 	defer res.Body.Close()
 
-	md, err := ioutil.ReadAll(res.Body)
+	md, err := io.ReadAll(res.Body)
 	if err != nil {
 		return nil, err
 	}
@@ -210,5 +208,5 @@ func main() {
 	}
 
 	userKeyList := getUserKeys(username, instanceAttributes, projectAttributes)
-	fmt.Printf(strings.Join(userKeyList, "\n"))
+	fmt.Print(strings.Join(userKeyList, "\n"))
 }
