@@ -1,16 +1,16 @@
-//  Copyright 2020 Google Inc. All Rights Reserved.
-//
-//  Licensed under the Apache License, Version 2.0 (the "License");
-//  you may not use this file except in compliance with the License.
-//  You may obtain a copy of the License at
-//
-//      http://www.apache.org/licenses/LICENSE-2.0
-//
-//  Unless required by applicable law or agreed to in writing, software
-//  distributed under the License is distributed on an "AS IS" BASIS,
-//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-//  See the License for the specific language governing permissions and
-//  limitations under the License.
+// Copyright 2020 Google LLC
+
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+
+//     https://www.apache.org/licenses/LICENSE-2.0
+
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 
 package main
 
@@ -20,58 +20,51 @@ import (
 	"os"
 	"time"
 
+	"github.com/GoogleCloudPlatform/guest-agent/google_guest_agent/run"
 	sspb "github.com/GoogleCloudPlatform/guest-agent/google_guest_agent/snapshot_service"
 	"github.com/GoogleCloudPlatform/guest-logging-go/logger"
 	"github.com/golang/groupcache/lru"
 	"google.golang.org/grpc"
+	"google.golang.org/grpc/credentials/insecure"
 )
 
 var (
 	scriptsDir                   = "/etc/google/snapshots/"
 	seenPreSnapshotOperationIds  = lru.New(128)
 	seenPostSnapshotOperationIds = lru.New(128)
+	maxRequestHandleAttempts     = 10 // completely arbitrary max attempt
 )
 
 type snapshotConfig struct {
 	timeout time.Duration // seconds
 }
 
-type invalidSnapshotConfig struct {
-	msg string
-}
-
-func (e *invalidSnapshotConfig) Error() string {
-	return fmt.Sprintf("invalid config: %s", e.msg)
-}
-
-func getSnapshotConfig() (snapshotConfig, error) {
+func getSnapshotConfig(timeoutInSeconds int) (snapshotConfig, error) {
 	var conf snapshotConfig
-	conf.timeout = time.Duration(config.Section("Snapshots").Key("timeout_in_seconds").MustInt(60)) * time.Second
-
+	conf.timeout = time.Duration(timeoutInSeconds) * time.Second
 	return conf, nil
 }
 
-func runScript(path, disks string, config snapshotConfig) (int, sspb.AgentErrorCode) {
+func runScript(ctx context.Context, path, disks string, config snapshotConfig) (int, sspb.AgentErrorCode) {
 	logger.Infof("Running guest consistent snapshot script at: %s.", path)
 
 	if _, err := os.Stat(path); os.IsNotExist(err) {
 		return -1, sspb.AgentErrorCode_SCRIPT_NOT_FOUND
 	}
 
-	execResult := runCmdOutputWithTimeout(config.timeout, path, disks)
-
-	if execResult.code == 124 {
-		return execResult.code, sspb.AgentErrorCode_SCRIPT_TIMED_OUT
+	execResult := run.WithOutputTimeout(ctx, config.timeout, path, disks)
+	if execResult.ExitCode == 124 {
+		return execResult.ExitCode, sspb.AgentErrorCode_SCRIPT_TIMED_OUT
 	}
 
-	if execResult.code != 0 {
-		return execResult.code, sspb.AgentErrorCode_UNHANDLED_SCRIPT_ERROR
+	if execResult.ExitCode != 0 {
+		return execResult.ExitCode, sspb.AgentErrorCode_UNHANDLED_SCRIPT_ERROR
 	}
 
-	return execResult.code, sspb.AgentErrorCode_NO_ERROR
+	return execResult.ExitCode, sspb.AgentErrorCode_NO_ERROR
 }
 
-func listenForSnapshotRequests(address string, requestChan chan<- *sspb.GuestMessage) {
+func listenForSnapshotRequests(ctx context.Context, address string, requestChan chan<- *sspb.GuestMessage) {
 	for {
 		// Start hanging connection on server that feeds to channel
 		logger.Infof("Attempting to connect to snapshot service at %s.", address)
@@ -82,7 +75,7 @@ func listenForSnapshotRequests(address string, requestChan chan<- *sspb.GuestMes
 		}
 
 		c := sspb.NewSnapshotServiceClient(conn)
-		ctx, cancel := context.WithCancel(context.Background())
+		ctx, cancel := context.WithCancel(ctx)
 		guestReady := sspb.GuestReady{
 			RequestServerInfo: false,
 		}
@@ -105,7 +98,7 @@ func listenForSnapshotRequests(address string, requestChan chan<- *sspb.GuestMes
 	}
 }
 
-func getSnapshotResponse(guestMessage *sspb.GuestMessage) *sspb.SnapshotResponse {
+func getSnapshotResponse(ctx context.Context, timeoutInSeconds int, guestMessage *sspb.GuestMessage) *sspb.SnapshotResponse {
 	switch {
 	case guestMessage.GetSnapshotRequest() != nil:
 		request := guestMessage.GetSnapshotRequest()
@@ -114,7 +107,7 @@ func getSnapshotResponse(guestMessage *sspb.GuestMessage) *sspb.SnapshotResponse
 			Type:        request.GetType(),
 		}
 
-		config, err := getSnapshotConfig()
+		config, err := getSnapshotConfig(timeoutInSeconds)
 		if err != nil {
 			response.AgentReturnCode = sspb.AgentErrorCode_INVALID_CONFIG
 			return response
@@ -146,7 +139,7 @@ func getSnapshotResponse(guestMessage *sspb.GuestMessage) *sspb.SnapshotResponse
 			return nil
 		}
 
-		scriptsReturnCode, agentErrorCode := runScript(url, request.GetDiskList(), config)
+		scriptsReturnCode, agentErrorCode := runScript(ctx, url, request.GetDiskList(), config)
 		response.ScriptsReturnCode = int32(scriptsReturnCode)
 		response.AgentReturnCode = agentErrorCode
 
@@ -156,9 +149,9 @@ func getSnapshotResponse(guestMessage *sspb.GuestMessage) *sspb.SnapshotResponse
 	return nil
 }
 
-func handleSnapshotRequests(address string, requestChan <-chan *sspb.GuestMessage) {
+func handleSnapshotRequests(ctx context.Context, timeoutInSeconds int, address string, requestChan <-chan *sspb.GuestMessage) {
 	for {
-		conn, err := grpc.Dial(address, grpc.WithInsecure(), grpc.WithBlock())
+		conn, err := grpc.Dial(address, grpc.WithTransportCredentials(insecure.NewCredentials()), grpc.WithBlock())
 		if err != nil {
 			logger.Errorf("Failed to connect to snapshot service: %v.", err)
 			return
@@ -166,22 +159,36 @@ func handleSnapshotRequests(address string, requestChan <-chan *sspb.GuestMessag
 		for {
 			// Listen on channel and respond
 			guestMessage := <-requestChan
-			response := getSnapshotResponse(guestMessage)
-			for {
+			response := getSnapshotResponse(ctx, timeoutInSeconds, guestMessage)
+
+			// We either got a duplicated pre/post or an invalid request
+			// in both cases we want to ignore it.
+			if response == nil {
+				continue
+			}
+
+			for i := 0; i < maxRequestHandleAttempts; i++ {
+				logger.Infof("Attempt %d/%d of handling snapshot request.", i+1, maxRequestHandleAttempts)
+
 				c := sspb.NewSnapshotServiceClient(conn)
-				ctx, cancel := context.WithCancel(context.Background())
+				ctx, cancel := context.WithCancel(ctx)
+				defer cancel()
+
 				_, err = c.HandleResponsesFromGuest(ctx, response)
-				if err == nil {
-					cancel()
-					break
+				if err != nil {
+					logger.Errorf("Error sending response: %v.", err)
+					time.Sleep(1 * time.Second) // Avoid idle looping
+					continue
 				}
-				logger.Errorf("Error sending response: %v.", err)
+
+				logger.Debugf("Successfully handled snapshot request.")
+				break
 			}
 		}
 	}
 }
 
-func startSnapshotListener(snapshotServiceIP string, snapshotServicePort int) {
+func startSnapshotListener(ctx context.Context, snapshotServiceIP string, snapshotServicePort int, timeoutInSeconds int) {
 	requestChan := make(chan *sspb.GuestMessage)
 	address := fmt.Sprintf("%s:%d", snapshotServiceIP, snapshotServicePort)
 
@@ -192,6 +199,6 @@ func startSnapshotListener(snapshotServiceIP string, snapshotServicePort int) {
 		os.MkdirAll(scriptsDir, 0700)
 	}
 
-	go listenForSnapshotRequests(address, requestChan)
-	go handleSnapshotRequests(address, requestChan)
+	go listenForSnapshotRequests(ctx, address, requestChan)
+	go handleSnapshotRequests(ctx, timeoutInSeconds, address, requestChan)
 }
