@@ -1,16 +1,16 @@
-//  Copyright 2022 Google Inc. All Rights Reserved.
-//
-//  Licensed under the Apache License, Version 2.0 (the "License");
-//  you may not use this file except in compliance with the License.
-//  You may obtain a copy of the License at
-//
-//      http://www.apache.org/licenses/LICENSE-2.0
-//
-//  Unless required by applicable law or agreed to in writing, software
-//  distributed under the License is distributed on an "AS IS" BASIS,
-//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-//  See the License for the specific language governing permissions and
-//  limitations under the License.
+// Copyright 2022 Google LLC
+
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+
+//     https://www.apache.org/licenses/LICENSE-2.0
+
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 
 // Utilities for Google Guest Agent and Google Authorized Keys
 
@@ -19,6 +19,10 @@ package utils
 import (
 	"encoding/json"
 	"errors"
+	"fmt"
+	"io/fs"
+	"os"
+	"path/filepath"
 	"regexp"
 	"strings"
 	"time"
@@ -46,7 +50,7 @@ type sshExpiration struct {
 func CheckExpiredKey(key string) error {
 	trimmedKey := strings.Trim(key, " ")
 	if trimmedKey == "" {
-		return errors.New("Invalid ssh key entry - empty key")
+		return errors.New("invalid ssh key entry - empty key")
 	}
 	fields := strings.SplitN(trimmedKey, " ", 4)
 	if len(fields) < 3 {
@@ -55,7 +59,7 @@ func CheckExpiredKey(key string) error {
 	}
 	if len(fields) == 3 && fields[2] == "google-ssh" {
 		// expiring key without expiration format.
-		return errors.New("Invalid ssh key entry - expiration missing")
+		return errors.New("invalid ssh key entry - expiration missing")
 	}
 	if len(fields) >= 3 && fields[2] != "google-ssh" {
 		// Non-expiring key with an arbitrary comment part
@@ -72,7 +76,7 @@ func CheckExpiredKey(key string) error {
 			return err
 		}
 		if expired {
-			return errors.New("Invalid ssh key entry - expired key")
+			return errors.New("invalid ssh key entry - expired key")
 		}
 	}
 
@@ -99,13 +103,13 @@ func CheckExpired(expireOn string) (bool, error) {
 // Currently, the only banned characters are whitespace characters.
 func ValidateUser(user string) error {
 	if user == "" {
-		return errors.New("Invalid username - it is empty")
+		return errors.New("invalid username - it is empty")
 	}
 
-	whiteSpaceRegexp, _ := regexp.Compile("\\s")
+	whiteSpaceRegexp, _ := regexp.Compile(`\s`)
 
 	if whiteSpaceRegexp.MatchString(user) {
-		return errors.New("Invalid username - whitespace detected")
+		return errors.New("invalid username - whitespace detected")
 	}
 	return nil
 }
@@ -115,18 +119,18 @@ func ValidateUser(user string) error {
 func GetUserKey(rawKey string) (string, string, error) {
 	key := strings.Trim(rawKey, " ")
 	if key == "" {
-		return "", "", errors.New("Invalid ssh key entry - empty key")
+		return "", "", errors.New("invalid ssh key entry - empty key")
 	}
 	idx := strings.Index(key, ":")
 	if idx == -1 {
-		return "", "", errors.New("Invalid ssh key entry - unrecognized format. Expecting user:ssh-key")
+		return "", "", errors.New("invalid ssh key entry - unrecognized format. Expecting user:ssh-key")
 	}
 	user := key[:idx]
 	if user == "" {
-		return "", "", errors.New("Invalid ssh key entry - user missing")
+		return "", "", errors.New("invalid ssh key entry - user missing")
 	}
 	if key[idx+1:] == "" {
-		return "", "", errors.New("Invalid ssh key entry - key missing")
+		return "", "", errors.New("invalid ssh key entry - key missing")
 	}
 
 	return user, key[idx+1:], nil
@@ -160,3 +164,59 @@ func (s *SerialPort) Write(b []byte) (int, error) {
 
 	return p.Write(b)
 }
+
+// WriteFile creates parent directories if required and writes content to the output file.
+func WriteFile(content []byte, outputFile string, perm fs.FileMode) error {
+	if err := os.MkdirAll(filepath.Dir(outputFile), perm); err != nil {
+		return fmt.Errorf("unable to create required directories for %q: %w", outputFile, err)
+	}
+	return os.WriteFile(outputFile, content, perm)
+}
+
+// SaferWriteFile writes to a temporary file and then replaces the expected output file.
+// This prevents other processes from reading partial content while the writer is still writing.
+func SaferWriteFile(content []byte, outputFile string, perm fs.FileMode) error {
+	dir := filepath.Dir(outputFile)
+	name := filepath.Base(outputFile)
+
+	if err := os.MkdirAll(dir, perm); err != nil {
+		return fmt.Errorf("unable to create required directories %q: %w", dir, err)
+	}
+
+	tmp, err := os.CreateTemp(dir, name+"*")
+	if err != nil {
+		return fmt.Errorf("unable to create temporary file under %q: %w", dir, err)
+	}
+
+	if err := os.Chmod(tmp.Name(), perm); err != nil {
+		return fmt.Errorf("unable to set permissions on temporary file %q: %w", dir, err)
+	}
+
+	if err := tmp.Close(); err != nil {
+		return fmt.Errorf("failed to close temporary file: %w", err)
+	}
+
+	if err := WriteFile(content, tmp.Name(), perm); err != nil {
+		return fmt.Errorf("unable to write to a temporary file %q: %w", tmp.Name(), err)
+	}
+
+	return os.Rename(tmp.Name(), outputFile)
+}
+
+// CopyFile copies content from src to dst and sets permissions.
+func CopyFile(src, dst string, perm fs.FileMode) error {
+	b, err := os.ReadFile(src)
+	if err != nil {
+		return fmt.Errorf("failed to read %q: %w", src, err)
+	}
+
+	if err := WriteFile(b, dst, perm); err != nil {
+		return fmt.Errorf("failed to write %q: %w", dst, err)
+	}
+
+	if err := os.Chmod(dst, perm); err != nil {
+		return fmt.Errorf("unable to set permissions on destination file %q: %w", dst, err)
+	}
+
+	return nil
+}
